@@ -1,15 +1,9 @@
 from cro_validate.enum import DataType
-import cro_validate.api.configuration_api as ConfigApi
-import cro_validate.api.name_resolver_api as NameResolverApi
-import cro_validate.api.exception_api as ExceptionApi
-import cro_validate.api.example_api as ExampleApi
+from cro_validate.classes.configuration_classes import Config
 import cro_validate.classes.parameter_classes as Parameters
 import cro_validate.classes.schema_classes as Schemas
 import cro_validate.classes.name_strategy_classes as NameStrategies
-
-
-class DataDefinitionDefaultValue:
-	pass
+import cro_validate.classes.util_classes as Utils
 
 
 class Meta:
@@ -36,7 +30,6 @@ class DefaultMeta(Meta):
 class Definition:
 	def __init__(
 				self,
-				definition_index,
 				name,
 				aliases,
 				description,
@@ -53,7 +46,6 @@ class Definition:
 			):
 		self.name = name
 		self.aliases = aliases
-		self.definitions = definition_index
 		self.description = description
 		self.data_type = data_type
 		self.data_format = data_format
@@ -68,9 +60,9 @@ class Definition:
 		self.dependency_resolver = dependency_resolver
 		# Name
 		######
-		self.name = ConfigApi.get_definition_name_strategy().create_name(self, self.name)
+		self.name = Config.definition_name_strategy.create_name(self, self.name)
 		if self.name is None:
-			ExceptionApi.create_internal_error(
+			ConfigApi.exception_factory.create_input_error(
 					'<unset>', 'Definition name cannot be None (description={0})'.format(self.description)
 				)
 		# Nullable
@@ -96,29 +88,35 @@ class Definition:
 		# Examples
 		##########
 		if not self.examples:
-			self.examples = ExampleApi.get_default_examples(self)
+			self.examples = Config.default_examples_provider.get_examples(self)
 		if not self.is_object() and not self.is_array():
 			if not self.examples:
-				raise ExceptionApi.create_input_error(self.name, 'Missing examples')
+				raise Config.exception_factory.create_input_error(self.name, 'Missing examples')
 		# Meta
 		######
 		self.meta.initialize(self)
 
 	def _get_obj_model_validator(self):
+		display_name = self.data_format.model_name
+		if self.data_format.display_name is not None:
+			display_name = str(self.data_format.display_name)
 		validator = Schemas.ModelValidator(
-				self.data_format.model_name,
-				self.data_format.allow_unknown_fields
+				name=display_name,
+				allow_unknown_fields=self.data_format.allow_unknown_fields,
+				case_sensitive=self.data_format.case_sensitive
 			)
 		required = set()
 		optional = set()
 		ignored = set()
 		unvalidated = set()
 		definition_names = {}
+		input_names = {}
 		output_names = {}
 		dependencies = {}
+		default_values = {}
 		model = self.data_format.model
 		if model is None:
-			raise ExceptionApi.create_internal_error(self.data_format.model_name, 'Missing model')
+			raise Config.exception_factory.create_internal_error(self.data_format.model_name, 'Missing model')
 		for name in dir(model):
 			if name.startswith('_'):
 				continue			
@@ -137,19 +135,25 @@ class Definition:
 				definition_names[name] = field_definition.definition_name
 			if field_definition.output_name:
 				output_names[name] = field_definition.output_name
+			if field_definition.input_name:
+				input_names[name] = field_definition.input_name
+			if not isinstance(field_definition.default_value, Utils.DataDefinitionDefaultValue):
+				default_values[name] = field_definition.default_value
 			definition_name = name
 			if field_definition.definition_name is not None:
 				definition_name = field_definition.definition_name
 			if field_definition.unvalidated is not True:
-				dependencies[name] = self.definitions.get(definition_name).dependencies
+				dependencies[name] = Index.get(definition_name).dependencies
 		validator.add_spec(
 				required=required,
 				optional=optional,
 				ignored=ignored,
 				unvalidated=unvalidated,
 				definition_names=definition_names,
+				input_names=input_names,
 				output_names=output_names,
-				dependencies=dependencies)
+				dependencies=dependencies,
+				default_values=default_values)
 		return validator
 
 	def _get_obj_validator(self):
@@ -163,10 +167,10 @@ class Definition:
 
 	def _validate_array(self, results, field_fqn, field_name, definition, values, dependent_values):
 		if not isinstance(values, list):
-			raise ExceptionApi.create_input_error(field_fqn, 'Expected array, received: {0}'.format(type(values)))
+			raise Config.exception_factory.create_input_error(field_fqn, 'Expected array, received: {0}'.format(type(values)))
 		items = []
 		for entry in values:
-			item = self.definitions.validate_input(None, field_fqn, field_name, self.data_format, entry, dependent_values)
+			item = Index.validate_input(None, field_fqn, field_name, self.data_format, entry, dependent_values)
 			items.append(item[field_name])
 		results[field_name] = items
 
@@ -175,6 +179,14 @@ class Definition:
 
 	def validate(self, results, field_fqn, field_name, definition, value, dependent_values):
 		try:
+			if field_name is None:
+				field_name = self.name
+				#if self.data_type == DataType.Object:
+					#field_name = self.validator.model_validator.name
+			if field_fqn is None:
+				field_fqn = field_name
+				if self.data_type == DataType.Object:
+					field_fqn = self.validator.model_validator.name
 			normalized = Parameters.Index()
 			validator = self.validator
 			dependent_definition_name = self.dependency_resolver.get_dependent_definition(
@@ -182,7 +194,7 @@ class Definition:
 					dependent_values
 				)
 			if dependent_definition_name is not None:
-				self.definitions.validate_input(
+				Index.validate_input(
 						results,
 						field_fqn,
 						field_name,
@@ -201,18 +213,18 @@ class Definition:
 			results.update(normalized)
 		except Exception as ex:
 			if self.is_internal:
-				raise ExceptionApi.create_internal_error(ex.source, ex.message)
+				raise Config.exception_factory.create_internal_error(ex.source, ex.message)
 			else:
 				raise ex
 
 	def has_default_value(self):
-		if isinstance(self.default_value, DataDefinitionDefaultValue):
+		if isinstance(self.default_value, Utils.DataDefinitionDefaultValue):
 			return False
 		return True
 
 	def get_default_value(self, name):
 		if not self.has_default_value():
-			raise ExceptionApi.create_internal_error(self.name, 'No default value configured')
+			raise Config.exception_factory.create_internal_error(self.name, 'No default value configured')
 		return self.default_value
 
 	def get_name(self):
@@ -275,7 +287,7 @@ class OneOfResolver(DependentDefinitionResolver):
 	def _update_dependency_idx_order(self, fqn, keys):
 		diff = self._dependencies.difference(keys)
 		if len(diff) > 0:
-			raise ExceptionApi.create_internal_error(fqn, 'Permutation keys must match dependencies.')
+			raise Config.exception_factory.create_internal_error(fqn, 'Permutation keys must match dependencies.')
 		self._dependencies.update([k for k in keys])
 		for k in keys:
 			if k in self._dependency_order:
@@ -293,7 +305,7 @@ class OneOfResolver(DependentDefinitionResolver):
 			idx = idx[state]
 		state = permutation[self._dependency_order[-1]]
 		if state in idx:
-			raise ExceptionApi.create_internal_error(fqn, 'Cannot re-index a permutation.')
+			raise Config.exception_factory.create_internal_error(fqn, 'Cannot re-index a permutation.')
 		idx[state] = value
 
 	def _get_permutation_value(self, fqn, permutation):
@@ -302,13 +314,13 @@ class OneOfResolver(DependentDefinitionResolver):
 		for i in range(len(self._dependency_order)):
 			k = self._dependency_order[i]
 			if k not in permutation:
-				raise ExceptionApi.create_internal_error(
+				raise Config.exception_factory.create_internal_error(
 						fqn,
 						'k not in permutation ({0})'.fromat(permutation)
 					)
 			state = permutation[k]
 			if state not in idx:
-				raise ExceptionApi.create_internal_error(
+				raise Config.exception_factory.create_internal_error(
 						fqn,
 						'Unknown permutation value (k={0} p={2}).'.format(k, state, permutation)
 					)
@@ -336,29 +348,28 @@ class OneOfResolver(DependentDefinitionResolver):
 		return self._dependencies
 
 
+class Index:
+	_idx = {}
 
-class DefinitionIndex:
-	def __init__(self):
-		self._index = {}
-
-	def get(self, name):
-		resolved = NameResolverApi.resolve_definition_name(self._index, name)
+	def get(definition_or_name):
+		if isinstance(definition_or_name, Definition):
+			return definition_or_name
+		definition_name = str(definition_or_name)
+		resolved = Config.definition_name_resolver.resolve(Index._idx, definition_name)
 		if resolved is None:
-			raise ExceptionApi.create_internal_error(name, 'Definition name resolution failed (Unknown definition name).')
-		return self._index[resolved]
+			raise Config.exception_factory.create_input_error(definition_name, 'Definition name resolution failed (Unknown definition name).')
+		return Index._idx[resolved]
 
-	def exists(self, name):
-		resolved = NameResolverApi.resolve_definition_name(self._index, name)
+	def exists(name):
+		resolved = Config.definition_name_resolver.resolve(Index._idx, name)
 		if resolved is None:
 			return False
 		return True
 
-	def as_dict(self):
-		return self._index
+	def as_dict():
+		return Index._idx
 
 	def register_definition(
-				self,
-				definition_index,
 				name,
 				aliases,
 				description,
@@ -374,7 +385,6 @@ class DefinitionIndex:
 				dependency_resolver
 			):
 		definition = Definition(
-				definition_index=definition_index,
 				name=name,
 				aliases=aliases,
 				description=description,
@@ -397,71 +407,75 @@ class DefinitionIndex:
 		else:
 			names.update(aliases)
 		for entry in names:
-			if definition_name in self._index:
-				raise ExceptionApi.create_internal_error(definition_name, 'Input definiton already exists.')
+			if definition_name in Index._idx:
+				raise Config.exception_factory.create_internal_error(definition_name, 'Input definiton already exists.')
 		for entry in names:
-			self._index[entry] = definition
+			Index._idx[entry] = definition
 		return definition
 
-	def validate_inputs(self, validated, **kw):
+	def validate_inputs(validated, **kw):
 		results = Parameters.Index.ensure(validated)
 		for name in kw:
-			self.validate_input(results, name, name, name, kw[name])
+			Index.validate_input(results, name, name, name, kw[name])
 		return results
 
-	def validate_input(self, validated, field_fqn, field_name, definition_name, value, dependent_values={}):
+	def validate_input(validated, field_fqn, field_name, definition_or_name, value, dependent_values={}):
+		if isinstance(definition_or_name, Definition):
+			definition_name = definition_or_name.name
+		else:
+			definition_name = str(definition_or_name)
 		results = Parameters.Index.ensure(validated)
 		nullable = False
-		definition = self.get(definition_name)
-		nullable = self.is_nullable(definition_name)
+		definition = Index.get(definition_name)
+		nullable = Index.is_nullable(definition_name)
 		if value is None:
-			if nullable is True or NameResolverApi.is_definition_name_nullable(self._index, definition_name):
+			if nullable is True or Config.definition_name_resolver.is_nullable(Index._idx, definition_name):
 				results[field_name] = None
 			else:
-				raise ExceptionApi.create_input_error(field_name, 'Not nullable.')
+				raise Config.exception_factory.create_input_error(field_name, 'Not nullable.')
 		else:
 			if not definition.validator:
-				raise ExceptionApi.create_internal_error(definition_name, "Missing validator.")
+				raise Config.exception_factory.create_internal_error(definition_name, "Missing validator.")
 			definition.validate(results, field_fqn, field_name, definition, value, dependent_values)
 		return results
 
-	def is_nullable(self, name):
+	def is_nullable(name):
 		nullable = False
-		if NameResolverApi.is_definition_name_nullable(self.as_dict(), name):
+		if Config.definition_name_resolver.is_nullable(Index.as_dict(), name):
 			nullable = True
 		else:
-			definition = self.get(name)
+			definition = Index.get(name)
 			nullable = definition.nullable
 		return nullable
 
-	def get_description(self, name, delim):
-		definition = self.get(name)
+	def get_description(name, delim):
+		definition = Index.get(name)
 		result = definition.description
 		if definition.rules is not None and len(definition.rules) > 0:
 			result = result + delim + delim.join([rule.get_description() for rule in definition.rules])
 		return result
 
-	def ensure_alias(self, name, alias):
-		definition = self.get(name)
-		if alias not in self._index:
-			self._index[alias] = definition
+	def ensure_alias(name, alias):
+		definition = Index.get(name)
+		if alias not in Index._idx:
+			Index._idx[alias] = definition
 
-	def list_definitions(self):
-		result = [k for k in self._index]
+	def list_definitions():
+		result = [k for k in Index._idx]
 		result.sort()
 		return result
 
-	def list_dependent_definitions(self, definition_name):
+	def list_dependent_definitions(definition_name):
 		results = set()
-		definition = self.get(definition_name)
+		definition = Index.get(definition_name)
 		if definition.data_type == DataType.Object:
 			result = definition.validator.list_definition_names()
 		elif definition.data_type == DataType.Array:
-			results = DefinitionIndex.list_dependent_definitions(definition.data_format)
+			results = Index.list_dependent_definitions(definition.data_format)
 		return results
 
-	def list_fields(self, name):
-		definition = self.get(name)
+	def list_fields(name):
+		definition = Index.get(name)
 		if definition.data_type == DataType.Object:
 			return definition.validator.list_field_names()
 		return [name]
\ No newline at end of file
