@@ -1,16 +1,17 @@
 from cro_validate.enum import DataType, SchemaOutputTarget
-import cro_validate.api.definition_api as DefinitionApi
-import cro_validate.api.name_api as NameApi
-import cro_validate.api.exception_api as ExceptionApi
+import cro_validate.classes.definition_classes as Definitions
+from cro_validate.classes.configuration_classes import Config
 import cro_validate.classes.parameter_classes as Parameters
-
+import cro_validate.classes.util_classes as Utils
 
 class Schema:
 	def __init__(
 				self,
 				model,
 				output_target=SchemaOutputTarget.Self,
-				allow_unknown_fields=False
+				allow_unknown_fields=False,
+				display_name=None,
+				case_sensitive=True
 			):
 		self.model = model
 		if isinstance(self.model, type):
@@ -18,23 +19,29 @@ class Schema:
 		self.model_name = self.model.__class__.__name__
 		self.output_target = output_target
 		self.allow_unknown_fields = allow_unknown_fields
+		self.display_name = display_name
+		self.case_sensitive = case_sensitive
 
 
 class Field:
 	def __init__(
 				self,
+				input_name=None,
 				output_name=None,
 				definition_name=None,
 				required=True,
 				ignored=False,
 				unvalidated=False,
-				dependencies=set()):
+				dependencies=set(),
+				default_value=Utils.DataDefinitionDefaultValue()):
+		self.input_name = input_name
 		self.output_name = output_name
 		self.definition_name = definition_name
 		self.required = required
 		self.ignored = ignored
 		self.unvalidated = unvalidated
 		self.dependencies = dependencies
+		self.default_value = default_value
 
 
 class Validator:
@@ -60,7 +67,7 @@ class Validator:
 			results.update(model_result)
 
 		else:
-			raise ExceptionApi.create_internal_error(self.name, 'Invalid schema output configured: {0}'.format(self.output_target))
+			raise Config.exception_factory.create_internal_error(self.name, 'Invalid schema output configured: {0}'.format(self.output_target))
 
 	def __init__(self, name, output_target, model_validator):
 		self.name = name
@@ -75,8 +82,14 @@ class Validator:
 			return True
 		return False
 
-	def get_definition_name(self, field_name):
-		return self.model_validator.get_definition_name(field_name)
+	def get_field_definition_name(self, field_name):
+		return self.model_validator.get_field_definition_name(field_name)
+
+	def get_field_input_name(self, field_name):
+		return self.model_validator.get_field_input_name(field_name)
+
+	def get_field_input_display_name(self, field_name):
+		return self.model_validator.get_field_input_display_name(field_name)
 
 	def get_dependency_order(self):
 		return self.model_validator.get_dependency_order()
@@ -104,7 +117,7 @@ class Validator:
 			definition_name = field_name
 			if definition_name in self.model_validator.definition_names:
 				definition_name = self.model_validator.definition_names[field_name]
-			field = DefinitionApi.get(definition_name)
+			field = Definitions.Index.get(definition_name)
 			if field.data_type == DataType.Object:
 				results.update(field.validator.list_definition_names())
 				object_names.add(field_name)
@@ -118,56 +131,99 @@ class Validator:
 
 
 class ModelValidator:
-	def __init__(self, name, allow_unknown_fields):
+	def __init__(self, name, allow_unknown_fields, case_sensitive):
 		self.name = name
 		self.allow_unknown_fields = allow_unknown_fields
+		self.case_sensitive = case_sensitive
 		self.required = set()
 		self.optional = set()
 		self.ignored = set()
 		self.unvalidated = set()
 		self.definition_names = {}
+		self.input_field_names = {}
+		self.input_value_names = {}
+		self.default_values = {}
+		self.input_field_display_names = {}
 		self.output_names = {}
 		self.dependencies = {}
 		self.dependency_order = None
 
-	def get_definition_name(self, field_name):
+	def get_field_definition_name(self, field_name):
 		if field_name in self.definition_names:
 			return self.definition_names[field_name]
 		return field_name
 
+	def get_field_input_name(self, field_name):
+		if field_name in self.input_field_names:
+			return self.input_field_names[field_name]
+		return field_name
+
 	def get_dependency_order(self):
 		self._update_dependency_order()
 		return self.dependency_order
 
-	def _list_input_entries(self, vector):
-		if isinstance(vector, dict):
-			return [k for k in vector]
+	def get_field_input_display_name(self, field_name):
+		if field_name in self.input_field_names:
+			return self.input_field_display_names[field_name]
+		return field_name
+
+	def _list_input_field_names(self, vector):
 		result = []
-		for attribute_name in dir(vector):
-			try:
-				if attribute_name.startswith('_'):
-					continue
-				if not hasattr(vector, attribute_name): # possible dangling attr name if someone smashed the vector
-					continue
-				if callable(getattr(vector, attribute_name)):
-					continue
-				result.append(attribute_name)
-			except:
-				pass
+		if isinstance(vector, dict):
+			for k in vector:
+				if self.case_sensitive:
+					k = k.lower()
+				if k in self.input_value_names:
+					result.append(self.input_value_names[k])
+				else:
+					result.append(k)
+		else:
+			for attribute_name in dir(vector):
+				try:
+					if attribute_name.startswith('_'):
+						continue
+					if not hasattr(vector, attribute_name): # possible dangling attr name if someone smashed the vector
+						continue
+					if callable(getattr(vector, attribute_name)):
+						continue
+					if self.case_sensitive:
+						attribute_name = attribute_name.lower()
+					if attribute_name in self.input_value_names:
+						result.append(self.input_value_names[attribute_name])
+					else:
+						result.append(attribute_name)
+				except:
+					pass
 		return result
 
-	def _get_input_entry(self, field_name, source):
+	def _get_input_value(self, fqn, field_name, source):
+		value_name = field_name
+		if value_name in self.input_field_names:
+			value_name = self.input_field_names[field_name]
 		if isinstance(source, dict):
-			return source[field_name]
-		return getattr(source, field_name)
+			if value_name in source:
+				return source[value_name]
+		elif hasattr(source, value_name):
+			return getattr(source, value_name)
+		if field_name in self.default_values:
+			return self.default_values[field_name]
+		raise Config.exception_factory.create_input_error(fqn, 'Missing required value.')
 
-	def _set_input_entry(self, field_name, target, value):
+	def _get_output_value(self, value_name, source):
+		if isinstance(source, dict):
+			return source[value_name]
+		return getattr(source, value_name)
+
+	def _set_output_value(self, field_name, target, value):
+		value_name = field_name
+		if field_name in self.input_field_names:
+			value_name = self.input_field_names[field_name]
 		if isinstance(target, dict):
-			target[field_name] = value
+			target[value_name] = value
 		else:
-			old_val = getattr(target, field_name)
+			old_val = getattr(target, value_name)
 			if old_val != value: # avoid overwriting read-only values
-				setattr(target, field_name, value)
+				setattr(target, value_name, value)
 		return value
 
 	def _rename_output(self, src_field_name, target_field_name, target):
@@ -180,18 +236,16 @@ class ModelValidator:
 			delattr(target, src_field_name)
 			setattr(target, target_field_name, tmp)
 
-	def _validate(self, validated, fqn, field_name, values, callback):
+	def _validate(self, validated, fqn, values, callback):
 		result = Parameters.Index.ensure(validated)
 		if isinstance(values, dict):
 			normalized = Parameters.Index(values)
 		else:
 			normalized = values
-		field_names = self._list_input_entries(values)
+		field_names = self._list_input_field_names(normalized)
 		kw = Parameters.Index()
 		missing = set(self.required)
-
 		self._update_dependency_order()
-
 		# Build kw
 		##########
 		for field_name in field_names:
@@ -199,26 +253,29 @@ class ModelValidator:
 				missing.remove(field_name)
 			elif field_name not in self.optional:
 				if not self.allow_unknown_fields:
-					raise ExceptionApi.create_input_error(NameApi.get_fqn(fqn, field_name), 'Unknown field.')
+					display_name = self.get_field_input_display_name(field_name)
+					raise Config.exception_factory.create_input_error(Config.definition_name_strategy.get_fqn(fqn, display_name), 'Unknown field.')
 				else:
 					continue
 			if field_name not in self.ignored:
-				kw[field_name] = self._get_input_entry(field_name, normalized)
-
+				kw[field_name] = self._get_input_value(fqn, field_name, normalized)
 		# Validate
 		##########
 		if len(missing) > 0:
-			raise ExceptionApi.create_input_error(self.name, 'Missing required values: {0}'.format(', '.join(missing)))
+			display_names = []
+			for entry in missing:
+				display_names.append(self.get_field_input_display_name(entry))
+			raise Config.exception_factory.create_input_error(self.name, 'Missing required values: {0}'.format(', '.join(display_names)))
 		for entry in self.dependency_order:
 			# Unvalidated
 			#############
 			if entry in self.unvalidated:
-				result[entry] = self._get_input_entry(entry, normalized)
+				result[entry] = self._get_input_value(fqn, entry, normalized)
 				continue
 			# Default Value
 			###############
-			field_definition_name = self.get_definition_name(entry)
-			field_definition = DefinitionApi.get(field_definition_name)
+			field_definition_name = self.get_field_definition_name(entry)
+			field_definition = Definitions.Index.get(field_definition_name)
 			if entry not in kw:
 				if field_definition.has_default_value():
 					result[entry] = field_definition.get_default_value(entry)
@@ -227,27 +284,27 @@ class ModelValidator:
 			##############
 			dependencies = field_definition.dependencies
 			dependent_values = {k:kw[k] for k in kw if k in dependencies}
-			child_fqn = NameApi.get_fqn(fqn, entry)
+			display_name = self.get_field_input_display_name(entry)
+			child_fqn = Config.definition_name_strategy.get_fqn(fqn, display_name)
 			definition_result = callback(
 					validated=None,
 					field_fqn=child_fqn,
 					field_name=entry,
-					definition_name=field_definition_name,
+					definition_or_name=field_definition_name,
 					value=kw[entry],
 					dependent_values=dependent_values)
 			for definition_result_entry in definition_result:
-				self._set_input_entry(
+				self._set_output_value(
 						definition_result_entry,
 						result,
-						self._get_input_entry(
+						self._get_output_value(
 								definition_result_entry,
 								definition_result
 							)
 					)
-
 		# Output Names
 		##############
-		result_keys = self._list_input_entries(result)
+		result_keys = self._list_input_field_names(result)
 		for entry in result_keys:
 			if entry in self.output_names:
 				output_name = self.output_names[entry]
@@ -267,13 +324,13 @@ class ModelValidator:
 		for k in self.required:
 			if k in self.unvalidated:
 				continue
-			definition_name = self.get_definition_name(k)
-			dependencies_idx[k] = DefinitionApi.get(definition_name).dependencies
+			definition_name = self.get_field_definition_name(k)
+			dependencies_idx[k] = Definitions.Index.get(definition_name).dependencies
 		for k in self.optional:
 			if k in self.unvalidated:
 				continue
-			definition_name = self.get_definition_name(k)
-			dependencies_idx[k] = DefinitionApi.get(definition_name).dependencies
+			definition_name = self.get_field_definition_name(k)
+			dependencies_idx[k] = Definitions.Index.get(definition_name).dependencies
 		remaining = set([k for k in dependencies_idx.keys()])
 		result = []
 		# Visit
@@ -282,7 +339,7 @@ class ModelValidator:
 			if k not in remaining:
 				return
 			if k in branch:
-				raise ExceptionApi.create_internal_error(self.name, 'Dependency circular reference.')
+				raise Config.exception_factory.create_internal_error(self.name, 'Dependency circular reference.')
 			branch.add(k)
 			for edge in dependencies_idx[k]:
 				visit(edge, branch)
@@ -323,36 +380,61 @@ class ModelValidator:
 	def add_output_names(self, output_names):
 		self.output_names.update(output_names)
 
+	def add_input_names(self, input_names):
+		normalized = input_names
+		if not self.case_sensitive:
+			normalized = {k.lower():input_names[k].lower() for k in input_names}
+		self.input_field_display_names.update(input_names)
+		self.input_field_names.update(normalized)
+		self.input_value_names.update({normalized[k]:k for k in normalized})
+
 	def add_dependencies(self, new_dependencies):
 		for name in new_dependencies:
 			if name not in self.dependencies:
-				raise ExceptionApi.create_internal_error(str(add_dependencies) + '.' + str(name), 'Unknown field')
+				raise Config.exception_factory.create_internal_error(str(add_dependencies) + '.' + str(name), 'Unknown field')
 			self.dependencies[name].update(new_dependencies[name])
 		self.dependency_order = None
 
-	def add_spec(self, required, optional, ignored, unvalidated, definition_names={}, output_names={}, dependencies={}):
+	def add_default_values(self, default_values):
+		for field_name in default_values:
+			value = default_values[field_name]
+			if isinstance(value, Utils.DataDefinitionDefaultValue):
+				continue
+			self.default_values[field_name] = value
+
+	def add_spec(self,
+				required,
+				optional,
+				ignored,
+				unvalidated,
+				definition_names={},
+				input_names={},
+				output_names={},
+				dependencies={},
+				default_values={}
+			):
+		if self.case_sensitive:
+			required = {k.lower() for k in required}
+			optional = {k.lower() for k in optional}
+			ignored = {k.lower() for k in ignored}
+			unvalidated = {k.lower() for k in unvalidated}
+			ignored = {k.lower() for k in ignored}
+			dependencies = {k.lower():{k1.lower() for k1 in dependencies[k]} for k in dependencies}
+			default_values = {k.lower():default_values[k] for k in default_values}
 		self.add_required(required)
 		self.add_optional(optional)
 		self.add_ignored(ignored)
 		self.add_unvalidated(unvalidated)
 		self.add_definition_names(definition_names)
+		self.add_input_names(input_names)
 		self.add_output_names(output_names)
 		self.add_dependencies(dependencies)
+		self.add_default_values(default_values)
 
-	def validate_values(self, validated, fqn, field_name, definition, values):
+	def __call__(self, validated, fqn, field_name, definition, values):
 		return self._validate(
 				validated,
 				fqn,
-				field_name,
 				values,
-				DefinitionApi.validate_input,
-			)
-
-	def __call__(self, validated, fqn, field_name, definition, values):
-		return self.validate_values(
-				validated,
-				fqn,
-				field_name,
-				definition,
-				values
+				Definitions.Index.validate_input
 			)
\ No newline at end of file
